@@ -78,19 +78,15 @@ static gboolean is_enabled(struct btd_debug_desc *desc)
 	if (enabled == NULL)
 		return 0;
 
-	for (i = 0; enabled[i] != NULL; i++) {
-		if (desc->name != NULL && g_pattern_match_simple(enabled[i],
-							desc->name) == TRUE)
-			return 1;
+	for (i = 0; enabled[i] != NULL; i++)
 		if (desc->file != NULL && g_pattern_match_simple(enabled[i],
 							desc->file) == TRUE)
 			return 1;
-	}
 
 	return 0;
 }
 
-void __btd_toggle_debug()
+void __btd_toggle_debug(void)
 {
 	struct btd_debug_desc *desc;
 
@@ -102,30 +98,20 @@ void __btd_log_init(const char *debug, int detach)
 {
 	int option = LOG_NDELAY | LOG_PID;
 	struct btd_debug_desc *desc;
-	const char *name = NULL, *file = NULL;
 
 	if (debug != NULL)
 		enabled = g_strsplit_set(debug, ":, ", 0);
 
-	for (desc = __start___debug; desc < __stop___debug; desc++) {
-		if (file != NULL || name != NULL) {
-			if (g_strcmp0(desc->file, file) == 0) {
-				if (desc->name == NULL)
-					desc->name = name;
-			} else
-				file = NULL;
-		}
-
+	for (desc = __start___debug; desc < __stop___debug; desc++)
 		if (is_enabled(desc))
 			desc->flags |= BTD_DEBUG_FLAG_PRINT;
-	}
 
 	if (!detach)
 		option |= LOG_PERROR;
 
 	openlog("bluetoothd", option, LOG_DAEMON);
 
-	syslog(LOG_INFO, "Bluetooth deamon %s", VERSION);
+	syslog(LOG_INFO, "Bluetooth daemon %s", VERSION);
 }
 
 void __btd_log_cleanup(void)
